package cn.ibizlab.api.rest.extensions;

import cn.ibizlab.core.lite.domain.DstComponent;
import cn.ibizlab.core.lite.domain.DstConfig;
import cn.ibizlab.core.lite.domain.DstSystem;
import cn.ibizlab.core.lite.extensions.domain.*;
import cn.ibizlab.core.lite.extensions.service.DbEntityService;
import cn.ibizlab.core.lite.extensions.service.LiteCoreService;
import cn.ibizlab.core.lite.extensions.service.LiteModelService;
import cn.ibizlab.core.lite.service.IDstComponentService;
import cn.ibizlab.core.lite.service.IDstConfigService;
import cn.ibizlab.util.errors.BadRequestAlertException;
import cn.ibizlab.util.helper.DataObject;
import cn.ibizlab.util.security.AuthenticationUser;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.io.UnsupportedEncodingException;
import java.sql.Wrapper;
import java.util.Base64;
import java.util.List;

@Slf4j
@Api(tags = {"接口" })
@RestController("api-litecore")
@RequestMapping("")
public class LiteCoreResource {

    @Autowired
    private LiteModelService liteModelService;

    @Autowired
    private IDstComponentService dstComponentService;

    @Autowired
    @Lazy
    LiteCoreService liteCoreService;

    @Autowired
    private DbEntityService dbEntityService;

    @RequestMapping(method = RequestMethod.GET, value = "/lite/test/{modelid}")
    public ResponseEntity<List<ModelObj>> getstr(@PathVariable(name="modelid",required = false) String modelid)
    {

        if(StringUtils.isEmpty(modelid))
            modelid="dataModelff8080816a8b510c016a8c2f18b10001";
        List<ModelObj> list=dbEntityService.getModelObjs(modelid, DataObject.getBeginDate());
        return ResponseEntity.ok(list);
    }

    @RequestMapping(method = RequestMethod.GET, value = "/lite/{system}/entitys")
    public ResponseEntity<List<EntityModel>> getEntityModel(@PathVariable("system") String system) {
        return ResponseEntity.status(HttpStatus.OK).body(liteModelService.getEntityModel(system));
    }

    @RequestMapping(method = RequestMethod.GET, value = "/lite/{system}/entitys/{entity}")
    public ResponseEntity<EntityModel> getEntityModel(@PathVariable("system") String system,@PathVariable("entity") String entity) {
        return ResponseEntity.status(HttpStatus.OK).body(liteModelService.getEntityModel(system,entity));
    }

    @RequestMapping(method = RequestMethod.GET, value = "/lite/{system}/entitys/{entity}/fields")
    public ResponseEntity<List<FieldModel>> getEntityModelFields(@PathVariable("system") String system, @PathVariable("entity") String entity) {
        return ResponseEntity.status(HttpStatus.OK).body(liteModelService.getEntityModel(system,entity).getFields());
    }

    @RequestMapping(method = RequestMethod.GET, value = "/lite/{system}/entitys/{entity}/references")
    public ResponseEntity<List<RelationshipModel>> getEntityModelReferences(@PathVariable("system") String system, @PathVariable("entity") String entity) {
        return ResponseEntity.status(HttpStatus.OK).body(liteModelService.getEntityModel(system,entity).getReferences());
    }

    @RequestMapping(method = RequestMethod.GET, value = "/lite/{system}/entitys/{entity}/nesteds")
    public ResponseEntity<List<RelationshipModel>> getEntityModelNesteds(@PathVariable("system") String system, @PathVariable("entity") String entity) {
        return ResponseEntity.status(HttpStatus.OK).body(liteModelService.getEntityModel(system,entity).getNesteds());
    }

    @RequestMapping(method = RequestMethod.GET, value = "/lite/{app}/components/{component}")
    public ResponseEntity<JSON> getComponent(@PathVariable("app") String app, @PathVariable("component") String component) {

        DstComponent dstComponent = liteModelService.getComponent(app,component);

        return ResponseEntity.status(HttpStatus.OK).body(JSON.parseObject(dstComponent.getConfig()));
    }

    @ApiOperation(value = "保存系统", tags = {"系统" },  notes = "保存系统")
    @RequestMapping(method = RequestMethod.POST, value = "/lite/syncsysmodel")
    public ResponseEntity<Boolean> syncSysModel(@RequestBody SysModel sysModel) {
        return ResponseEntity.status(HttpStatus.OK).body(liteCoreService.syncSysModel(sysModel));
    }


    @RequestMapping(method = RequestMethod.GET, value = "/lite/sysapps")
    public ResponseEntity<List<DstSystem>> getSysApps() {
        return ResponseEntity.status(HttpStatus.OK).body(liteModelService.getSysApp());
    }


    @Autowired
    private IDstConfigService dstConfigService;


    @RequestMapping(method = RequestMethod.GET, value = { "/lite/mockdata/{configType}/{targetType}", "/lite/mockdata/{system}/{configType}/{targetType}","/lite/mockdata/{system}/{configType}/{targetType}/{userId}"})
    public ResponseEntity mockdata(@PathVariable(name = "system",required = false) String system,@PathVariable("configType") String configType, @PathVariable("targetType") String targetType,@PathVariable(name = "userId",required = false) String userId) {

        LambdaQueryWrapper<DstConfig> wrapper=Wrappers.lambdaQuery();
        if(!StringUtils.isEmpty(system))
            wrapper.eq(DstConfig::getSystemId,system);
        if(!StringUtils.isEmpty(userId))
            wrapper.eq(DstConfig::getUserId,userId);
        wrapper.eq(DstConfig::getCfgType,configType);
        wrapper.eq(DstConfig::getTargetType,targetType);
        DstConfig config=dstConfigService.getOne(wrapper,false);

        if(config==null)
            throw new BadRequestAlertException("未找到配置项","DstConfig",configType+"-"+targetType);

        return ResponseEntity.ok(JSON.parse(config.getCfg()));
    }


}
